/*
 * Copyright (C) 2016 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.cast.sample.mediaplayer;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.graphics.Point;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v7.widget.Toolbar;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.akamai.cast.ChromecastAmp;
import com.akamai.cast.activity.CastAmpCompatActivity;
import com.akamai.cast.utils.CastResources;
import com.akamai.cast.utils.Utils;
import com.akamai.media.VideoPlayerContainer;
import com.akamai.uimobile.generic.media.PlayerControlsOverlay;
import com.cast.sample.R;
import com.cast.sample.browser.VideoProvider;
import com.cast.sample.settings.CastPreference;
import com.cast.sample.ui.QueueListViewActivity;
import com.google.android.gms.cast.MediaInfo;
import com.google.android.gms.cast.MediaMetadata;

/**
 * Activity for the AMP media player.
 */
public class LocalPlayerActivity extends CastAmpCompatActivity {

    private static final String TAG = "LocalPlayerActivity";
    private static final float mAspectRatio = 72f / 128;
    private static final String AMP_LICENSE = "Sv7yaPhyan1qoaKbY9yP5rClbj8QNEQxHPmFoGGUxzqn5tSb7iRy6uJZhu_fUlG29W6vXmFpaqw8zf486ucOZA==";

    private VideoPlayerContainer playerContainer;
    private PlayerControlsOverlay mPlayerControlsOverlay;

    private TextView mTitleView;
    private TextView mDescriptionView;
    private TextView mAuthorView;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.player_activity);

        playerContainer = findViewById(R.id.videoView1);
        mPlayerControlsOverlay = findViewById(R.id.playerControls);
        setInlineUI();
        ChromecastAmp.init(this, playerContainer, mPlayerControlsOverlay, AMP_LICENSE, buildCastResources());

        mTitleView = (TextView) findViewById(R.id.titleTextView);
        mDescriptionView = (TextView) findViewById(R.id.descriptionTextView);
        mDescriptionView.setMovementMethod(new ScrollingMovementMethod());
        mAuthorView = (TextView) findViewById(R.id.authorTextView);

        // see what we need to play and where
        Bundle bundle = getIntent().getExtras();
        if (bundle != null) {
            setupActionBar();
            boolean shouldStartPlayback = bundle.getBoolean("shouldStart");
            int startPosition = bundle.getInt("startPosition", 0);
            MediaInfo selectedMedia = getIntent().getParcelableExtra("media");
            ChromecastAmp.shouldStartPlayback(selectedMedia, shouldStartPlayback, startPosition);
        }
    }

    private CastResources buildCastResources() {
        CastResources r = new CastResources();
        r.setMenuPopupDetachedQueue(R.menu.detached_popup_add_to_queue);
        r.setMenuPopupAddToQueue(R.menu.popup_add_to_queue);
        r.setIdActionPlayNow(R.id.action_play_now);
        r.setIdActionAddToQueue(R.id.action_add_to_queue);
        r.setIdActionPlayNext(R.id.action_play_next);
        r.setStringQueueItemAddedToPlayNext(R.string.queue_item_added_to_play_next);
        r.setStringQueueItemAddedToQueue(R.string.queue_item_added_to_queue);
        return r;
    }

    private void updateMetadata(boolean visible) {
        if (mTitleView == null) return;

        Point displaySize;
        if (!visible) {
            mDescriptionView.setVisibility(View.GONE);
            mTitleView.setVisibility(View.GONE);
            mAuthorView.setVisibility(View.GONE);
            displaySize = Utils.getDisplaySize(this);
            RelativeLayout.LayoutParams lp = new
                    RelativeLayout.LayoutParams(displaySize.x,
                    displaySize.y + getSupportActionBar().getHeight());
            lp.addRule(RelativeLayout.CENTER_IN_PARENT);
            playerContainer.setLayoutParams(lp);
            playerContainer.invalidate();
        } else {
            MediaInfo selectedMediaInfo = ChromecastAmp.getSelectedMediaInfo();
            if (selectedMediaInfo != null) {
                MediaMetadata mm = selectedMediaInfo.getMetadata();
                mDescriptionView.setText(selectedMediaInfo.getCustomData().optString(
                        VideoProvider.KEY_DESCRIPTION));
                mTitleView.setText(mm.getString(MediaMetadata.KEY_TITLE));
                mAuthorView.setText(mm.getString(MediaMetadata.KEY_SUBTITLE));
            }

            mDescriptionView.setVisibility(View.VISIBLE);
            mTitleView.setVisibility(View.VISIBLE);
            mAuthorView.setVisibility(View.VISIBLE);
            displaySize = Utils.getDisplaySize(this);
            RelativeLayout.LayoutParams lp = new
                    RelativeLayout.LayoutParams(displaySize.x,
                    (int) (displaySize.x * mAspectRatio));
            lp.addRule(RelativeLayout.BELOW, R.id.toolbar);
            playerContainer.setLayoutParams(lp);
            playerContainer.invalidate();
        }
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume() was called");
        if (playerContainer!=null) playerContainer.onResume();
        ChromecastAmp.onResume();
        super.onResume();
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause() was called");
        if (playerContainer!=null) playerContainer.onPause();
        ChromecastAmp.onPause();
        super.onPause();
    }

    @Override
    protected void onStop() {
        Log.d(TAG, "onStop() was called");
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy() is called");
        if (playerContainer!=null) playerContainer.onDestroy();
        super.onDestroy();
    }

    @Override
    protected void onStart() {
        Log.d(TAG, "onStart was called");
        super.onStart();
    }

    @Override
    public boolean dispatchKeyEvent(@NonNull KeyEvent event) {
        return ChromecastAmp.dispatchKeyEvent(event)
                || super.dispatchKeyEvent(event);
    }

    @SuppressLint("NewApi")
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        getSupportActionBar().show();
        if (newConfig.orientation == Configuration.ORIENTATION_LANDSCAPE) {
            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                    WindowManager.LayoutParams.FLAG_FULLSCREEN);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
                getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LOW_PROFILE);
            }
            updateMetadata(false);
        } else {
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN,
                    WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
            getWindow().clearFlags(
                    WindowManager.LayoutParams.FLAG_FULLSCREEN);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
                getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_VISIBLE);
            }
            updateMetadata(true);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.player, menu);
        ChromecastAmp.createMenuItem(this, menu, R.id.media_route_menu_item);
        MenuItem mQueueMenuItem = menu.findItem(R.id.action_show_queue);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        menu.findItem(R.id.action_show_queue).setVisible(
                ChromecastAmp.isConnected());

        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent intent;
        if (item.getItemId() == R.id.action_settings) {
            intent = new Intent(LocalPlayerActivity.this, CastPreference.class);
            startActivity(intent);
        } else if (item.getItemId() == R.id.action_show_queue) {
            intent = new Intent(LocalPlayerActivity.this, QueueListViewActivity.class);
            startActivity(intent);
        } else if (item.getItemId() == android.R.id.home) {
            ActivityCompat.finishAfterTransition(this);
        }
        return true;
    }

    private void setupActionBar() {
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setTitle(ChromecastAmp.getSelectedMetadataTitle());
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
    }

    private void validateFullScreen() {
        /*
        The fullscreen approach in this case is based on the current orientation
        so the App just forces a specific one and the config changes method handles the
        action accordingly
        */
        if (mPlayerControlsOverlay.isFullScreenMode()) {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT);
        } else {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE);
        }
    }

    /*
     * The PlayerControlsOverlay is a customized FrameLayout, that contains the required methods to
     * control the basic interaction between the user and a video player, play, pause, replay, scrubbing
     * DVR, etc.
     *
     * The whole point of having a generic UI is that the module is able to handle that basic interaction,
     * regardless of how the UI is displayed in the screen, so you can create yoy own UI as you do normally
     * with any activity or fragment in Android.
     *
     * In order to do that, the module needs as a requirement, the id of the xml that contains the definition of your
     * UI, this is a normal xml layout file, in here, you can create and place any button or UI widget
     * as you prefer.
     * */
    private void setFullScreenUI() {
        mPlayerControlsOverlay.overrideControlsLayout(R.layout.androidsdk_mediaplayercontroller_fullscreen);
        setupFullScreenPlayerControls();
    }

    private void setInlineUI() {
        mPlayerControlsOverlay.overrideControlsLayout(R.layout.androidsdk_mediaplayercontroller);
        setupInlinePlayerControls();
    }

    /*
     * Besides receiving the xml layout of your choosing, you need to let the Controls Overlay know
     * what functionality is going to be handled internally, that's why there is a set of methods available
     * for that.
     *
     * This is important because not only you have the freedom to create your own xml for the controls
     * but you are not restricted to names or specific conditions, you can create the component as you
     * wish, and then just pass the id reference of the specific component, and then the module
     * handles its functionality internally, so you don't have to
     * worry about handling the resources or icons for each button, call the correct player methods or
     * make sure the data displayed is correct, the module those that for you.
     * */
    private void setupFullScreenPlayerControls() {

        mPlayerControlsOverlay.managePlayPause(R.id.androidsdk_playPauseCtrl,
                R.mipmap.amp_play,
                R.mipmap.amp_pause);
        mPlayerControlsOverlay.manageCurrentPosition(R.id.androidsdk_seekbarTextCtrl);
        mPlayerControlsOverlay.manageTimeRemaining(R.id.video_duration);
        mPlayerControlsOverlay.manageScrubbing(R.id.androidsdk_seekbarCtrl,
                R.id.androidsdk_seekToLiveAction);
        mPlayerControlsOverlay.manageFullScreen(R.id.androidsdk_fullscreenCtrl,
                R.mipmap.amp_non_fullscreen_btn,
                R.mipmap.amp_fullscreen_btn);
    }

    private void setupInlinePlayerControls() {
        mPlayerControlsOverlay.managePlayPause(R.id.androidsdk_playPauseCtrl,
                R.mipmap.play,
                R.mipmap.pause);
        mPlayerControlsOverlay.manageCurrentPosition(R.id.androidsdk_seekbarTextCtrl);
        mPlayerControlsOverlay.manageTimeRemaining(R.id.video_duration);
        mPlayerControlsOverlay.manageScrubbing(R.id.androidsdk_seekbarCtrl,
                R.id.androidsdk_seekToLiveAction);

        mPlayerControlsOverlay.manageFullScreen(R.id.androidsdk_fullscreenCtrl,
                R.mipmap.amp_non_fullscreen_btn,
                R.mipmap.amp_fullscreen_btn);

    }

}
